<?php
/**
Update Manager class for private use
**/

if (!defined('SMUZ_UPGRADE_API_URL')) {
	define( 'SMUZ_UPGRADE_API_URL', 'http://updatesapp.pluginops.com' );
}


if ( ! class_exists( 'PluginOpsPageBuilder_Update_Manager' ) ) {

/**
 * Update manager object to manage upgrades of our premium products. 
 * Issue updates manage license key activation and verfication.
 * Inject updates information into the WordPress Plugin Update API
 * @package PluginOps
 */

class PluginOpsPageBuilder_Update_Manager {

	protected $version = null;

	protected $slug = null;

	protected $license_key = null;

	protected $product_id = null;

	protected $api_url = '';

	protected $plugin_url = 'http://google.com';

	protected $license_key_text = 'License Key';

	protected $submit_btn_text = 'Verify';

	protected $license_key_help = '';

	protected $display_admin_notice = true;

	protected $verify_key_request_text = 'Processing Request';

	protected $verify_key_request_processed_text = 'Key is activated..';

	protected $admin_notice_update_text = 'Hola! Would you like to receive automatic updates and unlock premium support? Please activate your copy.';

	protected $license_key_activated_box_text = 'The license key is activated. Site will receive plugin updates';

	function __construct( $plugin_slug, $product_id = null ) {

		if ( empty( $plugin_slug ) )
			throw new Exception( 'Plugin slug cannot be empty' , 1 );

		if ( $product_id == null )
			throw new Exception( 'Product ID cannot be null' , 1 );

		$this->product_id = $product_id;

		$this->slug = $plugin_slug;
		$this->api_url = SMUZ_UPGRADE_API_URL . '/api/v1/product/';
		$this->key_option_name = md5('smuz_upgrade_key_' . $this->product_id);
		$this->key_activated_option_name = md5('smuz_upgrade_key_activated_' . $this->product_id);
		
		$this->adminUpdateNoticeInit();
		$this->updatesInjectionInit();

		add_action( 'admin_init', array( $this, 'licenseKeyManagerInit' ) );

	}

	/**
     * Return license key status canceled or active etc. 
     * @param 
     * @return string|...
     */
	public function getLicenseKeyStatus() {
		
		$key = $this->getLicenseKey();
		$url = SMUZ_UPGRADE_API_URL . '/api/v2/key/status/' . $key;

		$resp = wp_remote_get( $url );
		$resp = json_decode( $resp['body'] );
		$retval = '';

		if ( ! $resp->result && $resp->code == 'e2' )
			return 'canceled';

		if ( ! $resp->result && $resp->code == 'e1' )
			return 'invalid';

		if ( ! $resp->result && $resp->code == 'e3' )
			return 'notactivated';

		if ( $resp->result && $resp->code == 's1' )
			return 'activated';

	}

	public function getLicenseKey() {
		return apply_filters( $this->key_option_name, get_option( $this->key_option_name ) );
	}
	public function isLicenseKeyActivated() {
		return apply_filters( $this->key_activated_option_name, get_option( $this->key_activated_option_name ) );
	}


	private function adminUpdateNoticeInit(){

		if ( $this->isLicenseKeyActivated() ) return null;
		if ( ! $this->display_admin_notice ) return null;

		add_action( 'admin_notices', array( $this, 'displayUpdateAdminNotices' ) );

	}

	public function displayUpdateAdminNotices(){
		echo sprintf( '<div class="update notice notice-warning is-dismissible"><p>%s</p></div>', $this->admin_notice_update_text );
	}

	private function updatesInjectionInit(){

		if ( ! $this->isLicenseKeyActivated() )
			return null;

		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check' ) );
		add_filter( 'plugins_api', array( $this, 'injectUpdateInformation' ), 10, 3 );
	}

	public function licenseKeyManagerInit() {

		$id_url = 'wp_ajax_' . $this->product_id;

		add_action( $id_url, array( $this, 'licenseKeyVerify' ) );
	
	}

	public function check( $updates ) {

		if ( empty ( $updates->checked ) ) return $updates;

		$product_info_url = $this->api_url . $this->product_id;

		$product_info_url = $product_info_url . '?domain_name=' . esc_url( get_bloginfo('url') );

		$response = wp_remote_get( $product_info_url );

		if ( is_array( $response ) ) {

			if ( $this->isJson( $response['body'] ) ) {

				$resp = json_decode( $response['body'] );

				if ( $resp->result === 'success' 
					&& $resp->code === '000' 
					&& $resp->product->status === 'active' ) {
					
					if ( version_compare( 
							$this->version, $resp->product->version, '<' )  ) {
						
						$obj = new stdClass();

						$obj->slug = dirname( $this->slug );

						$obj->plugin = $this->slug;

						$obj->new_version = $resp->product->version;

						$obj->url = $this->plugin_url;

						$domain = get_bloginfo('url');

						$key = $this->getLicenseKey();

						$download_url = SMUZ_UPGRADE_API_URL . '/api/v2/download/' . $key . '/' . $this->product_id . '.zip?domainname=' . esc_url( $domain );

						$obj->package = $download_url;
						
						$updates->response[ $this->slug ] = $obj;

					}

				}
	
			}

			
		}

		return $updates;

	}


	function injectUpdateInformation( $res, $action, $args ) {

		$slug = dirname( $this->slug );

		if ( $action === 'plugin_information' && $args->slug == $slug ) {

			$product_info_url = $this->api_url . $this->product_id;

			$response = wp_remote_get( $product_info_url );

			if ( is_array( $response ) ) {

				if ( $this->isJson( $response['body'] ) ) {

					$resp = json_decode( $response['body'] );

					if ( $resp->result === 'success' 
						&& $resp->code === '000' 
						&& $resp->product->status === 'active' ) {
						
						if ( version_compare( 
								$this->version, $resp->product->version, '<' )  ) {
							
							$product = $resp->product;
					
							$obj = new stdClass();

							$obj->name = $product->name;

							$obj->slug = $slug;

							$obj->version = $resp->product->version;

							$product->metadata = (array)$product->metadata;

							$obj->homepage = $product->metadata['url'];

							$obj->sections = array();

							$domain = get_bloginfo('url');

							$key = $this->getLicenseKey();

							$download_url = SMUZ_UPGRADE_API_URL . '/api/v2/download/' . $key . '/' . $this->product_id . '.zip?domainname=' . esc_url( $domain );

							$obj->download_link = $download_url;

							if ( $product->metadata['description'] !== '' )
								$obj->sections['description'] = $product->metadata['description'];

							if ( $product->metadata['changelog'] !== '' )
								$obj->sections['changelog'] = $product->metadata['changelog'];

							if ( $product->metadata['bannerImg'] !== '' ) {

								$obj->banners = array();
								
								$obj->banners['low'] = $product->metadata['bannerImg'];
								$obj->banners['high'] = $product->metadata['bannerImg'];
							
							}

							return $obj;

						}

					}
		
				}

			
		}

		}

		return false;
	}

	public function setVersion( $version ) {

		$this->version = $version;

		return $this;

	}

	/**
		Product $id should match the product id in backend database. If wrong product $id is used wrong update can be installed on user system.
	**/
	public function setProductId( $id ) {

		$this->product_id = $id;

		return $this;

	}

	public function setLicenseKey( $key ) {

		$this->license_key = $key;

		return $this;

	}

	public function setLicenseKeyText( $text ) {

		$this->license_key_text = $text;

		return $this;
	}

	public function setSubmitBtnText( $text ) {

		$this->submit_btn_text = $text;

		return $this;
	}

	public function setLicenseKeyHelpText( $text ) {

		$this->license_key_help = $text;

		return $this;
	}

	public function setAdminNoticeText( $text ) {

		$this->admin_notice_update_text = $text;

		return $this;
	}

	public function setDisplayAdminNotice( $opt = true ) {

		$this->display_admin_notice = $opt;

		return $this;
	}

	public function setLicenseKeyActivatedBoxText( $text ) {

		$this->license_key_activated_box_text = $text;

		return $this;

	}

	public function isJson($string) {
    	return ((is_string($string) &&
            (is_object(json_decode($string)) ||
            is_array(json_decode($string))))) ? true : false;
	}

	/**
	Use it only for testing on local servers. Warning
	**/
	public function allowUnSafeUrls( $allow = true ) {

		if ( $allow )
			add_filter( 'http_request_args', array( $this, 'filterSafeUrls' ) );

		return $this;
	
	}

	function filterSafeUrls( $args ) {

		$args['reject_unsafe_urls'] = false;
		
		return $args;
	
	}

	public function licenseKeyVerify() {
		
		/**
			Only admins allowed
		**/
		if ( ! current_user_can( 'manage_options' ) )
			die('admins only');
		
		//Wall against xss
		if ( ! wp_verify_nonce( $_POST['nonce'], 'smf-upgrade-nonce' ) )
			wp_die( 'Invalid Nonce' );


		$license_key = $_POST['smf-upgrade-key'];
		//v2 api
		$api2_url = SMUZ_UPGRADE_API_URL . '/api/v2/licensekey/verify/'. $license_key . '/product/' . $this->product_id ;

		$api2_url_store_key = SMUZ_UPGRADE_API_URL . '/api/v2/licensekey/activate/'. $license_key . '/product/' . $this->product_id ;

		$response = wp_remote_get( $api2_url );


		error_reporting(E_ALL);
		ini_set('display_errors', 1);

		if ( is_array( $response ) ) {

			if ( $this->isJson( $response['body'] ) ) {

				$resp = json_decode( $response['body'] );

				//1stnested if start
				if ( $resp->result === true ) {

					$site_url = get_bloginfo( 'url' );

					$activate_response = wp_remote_post( $api2_url_store_key, array(
							'headers' => array( 'Content-type' => 'application/json' ),
							'body' => json_encode( array( 'domain' => $site_url, 'plugin_slug' => $this->slug ) ),
							'method' => 'POST'
						) );

					
					$active_resp = json_decode( $activate_response['body'] );
					
					//nested if start
					if (  $active_resp->result === true ) {

						update_option( $this->key_option_name, $license_key );
						update_option( $this->key_activated_option_name, true );

						echo $activate_response['body'];
					} else {

						echo $activate_response['body'];

					} //nested if end

				} else {

					echo $response['body'];
				
				} //1stnested if end

			}

		}

		wp_die('');

	}

	public function getLicenseKeyUIHtmlCode() {

		if ( ! $this->isLicenseKeyActivated() )
			return $this->licenseKeyFormCode();
		else
			return $this->licenseKeyFormCodeActivated();

	}

	public function licenseKeyFormCodeActivated() {

		$s = sprintf( '<div id="sm-upgrade-form-cont"><div id="smf-license-key-field-cont"><label>%s<input type="text" readonly disabled value="%s" /></label><p id="smf-activated-text">%s</p></div>', $this->license_key_text,esc_attr( $this->getLicenseKey() ), $this->license_key_activated_box_text );

		return $s;		

	}

	public function licenseKeyFormCode() {
			$nonce = wp_create_nonce( 'smf-upgrade-nonce' );
			ob_start();
		?>

			<div id="sm-upgrade-form-cont">

				<div id="smf-mask-cont">
					<div class="smf-mask" style="position: absolute;width: 100%;height: 100%;z-index: 99999;opacity: 0.8;color: #fff;color: #fff;background: #000;display: none;top: 0;left: 0"><p style="font-family: monospace;font-size: 25px;text-align: center;margin-top: 25%;"><?php echo $this->verify_key_request_text ?></p>
						<div id="fountainG">
							<div id="fountainG_1" class="fountainG"></div>
							<div id="fountainG_2" class="fountainG"></div>
							<div id="fountainG_3" class="fountainG"></div>
							<div id="fountainG_4" class="fountainG"></div>
							<div id="fountainG_5" class="fountainG"></div>
							<div id="fountainG_6" class="fountainG"></div>
							<div id="fountainG_7" class="fountainG"></div>
							<div id="fountainG_8" class="fountainG"></div>
						</div>
					</div>
				</div>

				<form action="<?php echo admin_url( 'admin-ajax.php?action='.$this->product_id ) ?>" method="POST" id="sm-upgrade-form" style="width:50%; padding: 15px; background: #fff;">

					<div class="smf-upgrade-notices"></div>

					<p>Enter Your License key here to activate plugin to unlock premium features, premium support and updates. </p>

					<div class="smf-upgrade-field-cont">

						<label class="smf-upgrade-label" for="smf-upgrade-key"> <h3> <?php echo $this->license_key_text ?> </h3> </label>
						<input type="text" placeholder="Enter license key here" name="smf-upgrade-key" id="smf-upgrade-key" class="smf-upgrade-field" placeholder="" value="<?php echo $this->getLicenseKey() ?>" style='width: 250px;' />
						<p class="description smf-upgrade-help-tip"><?php $this->license_key_help ?></p>

					</div>

					<div class="smf-upgrade-field-cont">

						<input type="submit" value="<?php echo $this->submit_btn_text ?>" name="smf-upgrade-submit" class="smf-upgrade-submit-btn button button-primary" />

					</div>
					<br>
					<p> Your license key should look something like this : <span style="background:#e3e3e3; padding: 5px; color:#333;"> wxPocKQMWYhv6Pb5menoGnYvrjxIsSEJs4WXoywA1ZkdzPbCeI </span></p><br>
					<input type="hidden" name="nonce" value="<?php echo $nonce ?>" />

					<!-- If ajax is not working or javascript is disabled.-->
					<input type="hidden" name="redirect_to" value="<?php echo '//' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'] ?>" />

				</form>

				<script type="text/javascript">
					jQuery(document).ready( function($) {

						$('#sm-upgrade-form').submit( function( e ){

							e.preventDefault();

							$ajax_url = $(this).prop( 'action' );


							$('body').append( $('#smf-mask-cont').html() );
							$('#smf-mask-cont').html( '' );
							$('.smf-mask').show();

							$.post( $ajax_url, $(this).serialize(), function( resp ) {
								
								resp = JSON.parse( resp );
				
								if ( resp.result == false ) {
									$('.smf-upgrade-notices').html( '<p> * '+resp.message + '</p>');
									$('.smf-upgrade-notices').addClass('notice error is-dismissible');
								} else {

									window.location.href = window.location.href;
									$('.smf-upgrade-notices').html('');
								}

							}).done( function() {
								setTimeout(function(){ $('.smf-mask').hide(); }, 500 );
							});

						} );

					});
				</script>
				<style>#fountainG{position:relative;width:234px;height:28px;margin:auto}.fountainG{position:absolute;top:0;background-color:#000;width:28px;height:28px;animation-name:bounce_fountainG;-o-animation-name:bounce_fountainG;-ms-animation-name:bounce_fountainG;-webkit-animation-name:bounce_fountainG;-moz-animation-name:bounce_fountainG;animation-duration:1.5s;-o-animation-duration:1.5s;-ms-animation-duration:1.5s;-webkit-animation-duration:1.5s;-moz-animation-duration:1.5s;animation-iteration-count:infinite;-o-animation-iteration-count:infinite;-ms-animation-iteration-count:infinite;-webkit-animation-iteration-count:infinite;-moz-animation-iteration-count:infinite;animation-direction:normal;-o-animation-direction:normal;-ms-animation-direction:normal;-webkit-animation-direction:normal;-moz-animation-direction:normal;transform:scale(.3);-o-transform:scale(.3);-ms-transform:scale(.3);-webkit-transform:scale(.3);-moz-transform:scale(.3);border-radius:19px;-o-border-radius:19px;-ms-border-radius:19px;-webkit-border-radius:19px;-moz-border-radius:19px}#fountainG_1{left:0;animation-delay:.6s;-o-animation-delay:.6s;-ms-animation-delay:.6s;-webkit-animation-delay:.6s;-moz-animation-delay:.6s}#fountainG_2{left:29px;animation-delay:.75s;-o-animation-delay:.75s;-ms-animation-delay:.75s;-webkit-animation-delay:.75s;-moz-animation-delay:.75s}#fountainG_3{left:58px;animation-delay:.9s;-o-animation-delay:.9s;-ms-animation-delay:.9s;-webkit-animation-delay:.9s;-moz-animation-delay:.9s}#fountainG_4{left:88px;animation-delay:1.05s;-o-animation-delay:1.05s;-ms-animation-delay:1.05s;-webkit-animation-delay:1.05s;-moz-animation-delay:1.05s}#fountainG_5{left:117px;animation-delay:1.2s;-o-animation-delay:1.2s;-ms-animation-delay:1.2s;-webkit-animation-delay:1.2s;-moz-animation-delay:1.2s}#fountainG_6{left:146px;animation-delay:1.35s;-o-animation-delay:1.35s;-ms-animation-delay:1.35s;-webkit-animation-delay:1.35s;-moz-animation-delay:1.35s}#fountainG_7{left:175px;animation-delay:1.5s;-o-animation-delay:1.5s;-ms-animation-delay:1.5s;-webkit-animation-delay:1.5s;-moz-animation-delay:1.5s}#fountainG_8{left:205px;animation-delay:1.64s;-o-animation-delay:1.64s;-ms-animation-delay:1.64s;-webkit-animation-delay:1.64s;-moz-animation-delay:1.64s}@keyframes bounce_fountainG{0%{transform:scale(1);background-color:#000}100%{transform:scale(.3);background-color:#fff}}@-o-keyframes bounce_fountainG{0%{-o-transform:scale(1);background-color:#000}100%{-o-transform:scale(.3);background-color:#fff}}@-ms-keyframes bounce_fountainG{0%{-ms-transform:scale(1);background-color:#000}100%{-ms-transform:scale(.3);background-color:#fff}}@-webkit-keyframes bounce_fountainG{0%{-webkit-transform:scale(1);background-color:#000}100%{-webkit-transform:scale(.3);background-color:#fff}}@-moz-keyframes bounce_fountainG{0%{-moz-transform:scale(1);background-color:#000}100%{-moz-transform:scale(.3);background-color:#fff}}</style>

			</div>

		<?php
		$output = ob_get_contents();
		ob_end_clean();
		return $output;
	}

}

}